@@ -1,5 +1,8 @@
 """Tests for flake8's utils module."""
+import io
+import logging
 import os
+import sys
 
 import mock
 import pytest
@@ -22,11 +25,7 @@ RELATIVE_PATHS = ["flake8", "pep8", "pyflakes", "mccabe"]
     ("E123,W234,,E206,,", ["E123", "W234", "E206"]),
     ("E123, W234,, E206,,", ["E123", "W234", "E206"]),
     ("E123,,W234,,E206,,", ["E123", "W234", "E206"]),
-    (["E123", "W234", "E206"], ["E123", "W234", "E206"]),
-    (["E123", "\n\tW234", "\n    E206"], ["E123", "W234", "E206"]),
-    (["E123", "\n\tW234", "\n    E206", "\n"], ["E123", "W234", "E206"]),
-    (["E123", "\n\tW234", "", "\n    E206", "\n"], ["E123", "W234", "E206"]),
-    (["E123", "\n\tW234", "", "\n    E206", ""], ["E123", "W234", "E206"]),
+    ("", []),
 ])
 def test_parse_comma_separated_list(value, expected):
     """Verify that similar inputs produce identical outputs."""
@@ -132,14 +131,13 @@ def test_normalize_path(value, expected):
 
 
 @pytest.mark.parametrize("value,expected", [
-    ("flake8,pep8,pyflakes,mccabe", ["flake8", "pep8", "pyflakes", "mccabe"]),
-    ("flake8,\n\tpep8,\n  pyflakes,\n\n    mccabe",
+    (["flake8", "pep8", "pyflakes", "mccabe"],
         ["flake8", "pep8", "pyflakes", "mccabe"]),
-    ("../flake8,../pep8,../pyflakes,../mccabe",
+    (["../flake8", "../pep8", "../pyflakes", "../mccabe"],
         [os.path.abspath("../" + p) for p in RELATIVE_PATHS]),
 ])
 def test_normalize_paths(value, expected):
-    """Verify we normalize comma-separated paths provided to the tool."""
+    """Verify we normalizes a sequence of paths provided to the tool."""
     assert utils.normalize_paths(value) == expected
 
 
@@ -163,35 +161,68 @@ def test_fnmatch(filename, patterns, expected):
     assert utils.fnmatch(filename, patterns) is expected
 
 
-def test_fnmatch_returns_the_default_with_empty_default():
-    """The default parameter should be returned when no patterns are given."""
-    sentinel = object()
-    assert utils.fnmatch('file.py', [], default=sentinel) is sentinel
+@pytest.fixture
+def files_dir(tmpdir):
+    """Create test dir for testing filenames_from."""
+    with tmpdir.as_cwd():
+        tmpdir.join('a/b/c.py').ensure()
+        tmpdir.join('a/b/d.py').ensure()
+        tmpdir.join('a/b/e/f.py').ensure()
+        yield tmpdir
 
 
+def _normpath(s):
+    return s.replace('/', os.sep)
+
+
+def _normpaths(pths):
+    return {_normpath(pth) for pth in pths}
+
+
+@pytest.mark.usefixtures('files_dir')
 def test_filenames_from_a_directory():
     """Verify that filenames_from walks a directory."""
-    filenames = list(utils.filenames_from('src/flake8/'))
-    assert len(filenames) > 2
-    assert 'src/flake8/__init__.py' in filenames
+    filenames = set(utils.filenames_from(_normpath('a/b/')))
+    # should include all files
+    expected = _normpaths(('a/b/c.py', 'a/b/d.py', 'a/b/e/f.py'))
+    assert filenames == expected
 
 
+@pytest.mark.usefixtures('files_dir')
 def test_filenames_from_a_directory_with_a_predicate():
     """Verify that predicates filter filenames_from."""
-    filenames = list(utils.filenames_from(
-        arg='src/flake8/',
-        predicate=lambda filename: filename == 'flake8/__init__.py',
+    filenames = set(utils.filenames_from(
+        arg=_normpath('a/b/'),
+        predicate=lambda path: path.endswith(_normpath('b/c.py')),
     ))
-    assert len(filenames) > 2
-    assert 'flake8/__init__.py' not in filenames
+    # should not include c.py
+    expected = _normpaths(('a/b/d.py', 'a/b/e/f.py'))
+    assert filenames == expected
 
 
+@pytest.mark.usefixtures('files_dir')
+def test_filenames_from_a_directory_with_a_predicate_from_the_current_dir():
+    """Verify that predicates filter filenames_from."""
+    filenames = set(utils.filenames_from(
+        arg=_normpath('./a/b'),
+        predicate=lambda path: path == 'c.py',
+    ))
+    # none should have matched the predicate so all returned
+    expected = _normpaths(('./a/b/c.py', './a/b/d.py', './a/b/e/f.py'))
+    assert filenames == expected
+
+
+@pytest.mark.usefixtures('files_dir')
 def test_filenames_from_a_single_file():
     """Verify that we simply yield that filename."""
-    filenames = list(utils.filenames_from('flake8/__init__.py'))
+    filenames = set(utils.filenames_from(_normpath('a/b/c.py')))
+    assert filenames == {_normpath('a/b/c.py')}
 
-    assert len(filenames) == 1
-    assert ['flake8/__init__.py'] == filenames
+
+def test_filenames_from_a_single_file_does_not_exist():
+    """Verify that a passed filename which does not exist is returned back."""
+    filenames = set(utils.filenames_from(_normpath('d/n/e.py')))
+    assert filenames == {_normpath('d/n/e.py')}
 
 
 def test_filenames_from_exclude_doesnt_exclude_directory_names(tmpdir):
@@ -213,7 +244,7 @@ def test_parameters_for_class_plugin():
     """Verify that we can retrieve the parameters for a class plugin."""
     class FakeCheck(object):
         def __init__(self, tree):
-            pass
+            raise NotImplementedError
 
     plugin = plugin_manager.Plugin('plugin-name', object())
     plugin._plugin = FakeCheck
@@ -223,7 +254,7 @@ def test_parameters_for_class_plugin():
 def test_parameters_for_function_plugin():
     """Verify that we retrieve the parameters for a function plugin."""
     def fake_plugin(physical_line, self, tree, optional=None):
-        pass
+        raise NotImplementedError
 
     plugin = plugin_manager.Plugin('plugin-name', object())
     plugin._plugin = fake_plugin
@@ -268,3 +299,18 @@ MULTI_FILE_INFO = {
 def test_parse_unified_diff(diff, parsed_diff):
     """Verify that what we parse from a diff matches expectations."""
     assert utils.parse_unified_diff(diff) == parsed_diff
+
+
+def test_matches_filename_for_excluding_dotfiles():
+    """Verify that `.` and `..` are not matched by `.*`."""
+    logger = logging.Logger(__name__)
+    assert not utils.matches_filename('.', ('.*',), '', logger)
+    assert not utils.matches_filename('..', ('.*',), '', logger)
+
+
+@pytest.mark.xfail(sys.version_info < (3,), reason='py3+ only behaviour')
+def test_stdin_get_value_crlf():
+    """Ensure that stdin is normalized from crlf to lf."""
+    stdin = io.TextIOWrapper(io.BytesIO(b'1\r\n2\r\n'), 'UTF-8')
+    with mock.patch.object(sys, 'stdin', stdin):
+        assert utils.stdin_get_value.__wrapped__() == '1\n2\n'
